import draw.*;
import colors.*;
import geometry.*;

//to represent a list of cities in a city map
interface ILoCity{
  
  // count all entries in this list
  public int listSize();
  
  // combine this list with the given one
  public ILoCity concat(ILoCity that);
  
  // draw these cities on the given Canvas
  public boolean drawCities(Canvas c);
  
  // produce the remainder of this list - if possible
  public ILoCity advance();
  
  // draw the next leg of this path, if possible
  public boolean drawNext(Canvas c);

  
  // draw the next leg of this path, from the given city
  public boolean drawLineTo(City that, Canvas c);
}

//to represent an empty list of cities in a city map
class MtLoCity implements ILoCity{
  MtLoCity(){}
  
  // count all entries in this list
  public int listSize(){
    return 0;
  }
  
  // combine this list with the given one
  public ILoCity concat(ILoCity that){
    return that;
  }
  
  // draw this city on the given Canvas
  public boolean drawCities(Canvas c){
    return true;
  }
  
  // produce the remainder of this list - if possible
  public ILoCity advance(){
    return this;
  }
  
  // draw the next leg of this path, if possible
  public boolean drawNext(Canvas c){
    return true;
  }
  
  // draw the next leg of this path, from the given city
  public boolean drawLineTo(City that, Canvas c){
    return true;
  }
}

//to represent a nonempty list of of cities in a city map
class ConsLoCity implements ILoCity{
  City first;
  ILoCity rest;
  
  ConsLoCity(City first, ILoCity rest){
    this.first = first;
    this.rest = rest;
  }
  
  // count all entries in this list
  public int listSize(){
    return 1 + this.rest.listSize();
  }

  // combine this list with the given one
  public ILoCity concat(ILoCity that){
    return new ConsLoCity(this.first, this.rest.concat(that));
  }
  
  // draw this city on the given Canvas
  public boolean drawCities(Canvas c){
    return this.first.drawCity(c) && this.rest.drawCities(c);
  }
  
  // produce the remainder of this list - if possible
  public ILoCity advance(){
    return this.rest;
  }
  
  // draw the next leg of this path, if possible
  public boolean drawNext(Canvas c){
    return this.rest.drawLineTo(this.first, c);
  }
  
  // draw the next leg of this path, from the given city
  public boolean drawLineTo(City that, Canvas c){
    return this.first.drawLineToCity(that, c);
  }
}